package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

public class Customer extends AbstractBehavior<Customer.Message> {

    String name;
    ActorRef<AddressBook.Message> adrBook;
    ArrayList<String> gegenstaende = new ArrayList<>(Arrays.asList("Spülmaschine","Altes CD-Regal", "Goldbarren","20kg Hanteln", "Holzkohlegrill","Blumenerde"));

    public interface Message {}

    //Delivery Nachricht Typ
    public static final class DeliveryMessage implements Message {
        public final Paket paket;

        public DeliveryMessage(Paket paket) {
            this.paket = paket;
        }
    }

    //Pickup Nachricht Typ
    public static final class PickUpMessage implements Message {
        public final ActorRef<DeliveryCar.Message> truckReference;
        public final String someString;

        public PickUpMessage(ActorRef<DeliveryCar.Message> truckReference, String someString) {
            this.truckReference = truckReference;
            this.someString = someString;
        }
    }

    public record AddressHello(ActorRef<AddressBook.Message> book) implements Message {}
    public record AddressMessage(ActorRef<Customer.Message> addresse, ActorRef<DeliveryCar.Message> truckReference) implements Message {}

    private Customer(ActorContext<Message> context, String name){
        super(context);
        this.name = name;
    }

    public static Behavior<Customer.Message> create(String name) {
        return Behaviors.setup(context -> new Customer(context, name));
    }

    //Verarbeiten von den DeliveryMessage und PickUpMessage Nachrichten
    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(DeliveryMessage.class, this::onDeliveryMessage)
                .onMessage(AddressHello.class, this::onAddressHello)
                .onMessage(PickUpMessage.class, this::onPickUpMessage)
                .onMessage(AddressMessage.class, this::onAddressMessage)
                .build();
    }

    //Hier wird das Adressbuch gespeichert, nachdem es sich gemeldet hat
    private Behavior<Message> onAddressHello(AddressHello msg){
        this.adrBook = msg.book;
        return this;
    }

    //Verhalten, wenn Pickup Nachricht eingegangen ist
    //Wenn 80% getroffen wurden, wird eine Anfrage an das Adressbuch geschickt und die Antwort
    // an anderer Stelle (onAddressMessage) verarbeitet, um das Paket zu verschicken
    private Behavior<Message> onPickUpMessage(PickUpMessage msg){
        getContext().getLog().info("Kunde {}, will kein Paket abschicken CAR: {}", this.name, msg.truckReference.path().name());
        Random random = new Random();
        int number = random.nextInt(101);
        if(number <= 80){
            //truck wird hier übergeben, damit man im nächsten Schritt wieder auf ihn zugreifen kann.
            this.adrBook.tell(new AddressBook.CustomerMessage(getContext().getSelf(), this.name, msg.truckReference));

            //Methode um einen Paket zu versenden
            //über onAddressMessage wird dann Paket aufgenommen
            onAddressMessage(new AddressMessage(getContext().getSelf(), msg.truckReference));
        }else{
            getContext().getLog().info("Kunde {}, will kein Paket abschicken", this.name);
            msg.truckReference.tell(new DeliveryCar.PickupResponse(null));
        }
        return this;
    }

    //Adresse des Empfängers von dem Adressbuch empfangen und das Paket wird in den Truck geladen und verschickt
    private Behavior<Message> onAddressMessage(AddressMessage adr){
        Random random = new Random();
        int index = random.nextInt(gegenstaende.size());
        String gegenstand = gegenstaende.get(index);
        Paket paket = new Paket(gegenstand,this.name, adr.addresse);
        getContext().getLog().info("Paket mit dem Inhalt {} wurde an {} verschickt (CAR: {})", paket.inhalt, adr.addresse.path().name(), adr.truckReference.path().name());
        adr.truckReference.tell(new DeliveryCar.PickupResponse(paket));
        return this;
    }

    //Verhalten, wenn Paket eingegangen ist
    private Behavior<Message> onDeliveryMessage(DeliveryMessage pkt){
        getContext().getLog().info("Ich habe ein Paket von {} erhalten mit dem Inhalt: {} ", pkt.paket.absender, pkt.paket.inhalt);
        return this;
    }
}